const state = {
    loadingPacks: false,
    packs: [],
    selectedPack: null
}

const getters = {
    loadingPacks(state) {
        return state.loadingPacks
    },
    packs(state) {
        return state.packs
    },
    selectedPack(state) {
        return state.selectedPack
    }
}

const mutations = {
    setLoadingPacks(state, loading) {
        state.loadingPacks = !!loading
    },
    setPacks(state, packs) {
        state.packs = packs
    },
    setSelectedPack(state, packId) {
        if (!packId) {
            state.selectedPack = null
        }

        state.selectedPack = state.packs.find(p => p.id === packId)
    }
}

const actions = {
    initPacks({ commit }, packs) {
        commit('setPacks', packs)
        commit('setLoadingPacks')
    },
    selectPack({ commit }, packId) {
        commit('setSelectedPack', packId)
    },
    resetPack({ commit }) {
        commit('setLoadingPacks')
        commit('setPacks', [])
    }
}

export default {
    state,
    getters,
    actions,
    mutations
}