const state = {
    loadingPricelists: false,
    pricelists: [],
    customerPricelist: null,
    showPricelists: false
}

const getters = {
    loadingPricelists(state) {
        return state.loadingPricelists
    },
    pricelists(state) {
        return state.pricelists
    },
    customerPricelist(state) {
        return state.customerPricelist
    },
    pricelistsForProduct(state, getters) {
        let product = getters.itemToDiscount

        if (!product) {
            return []
        }

        let companyId = getters.selectedUser.company.id
        let pricelists = []

        for (let p of state.pricelists) {
            if (p.type !== 'sale') {
                continue
            }

            for (let v of p.versions) {
                for (let i of v.items) {
                    if (i.productId !== product.id && i.productTmplId !== product.tmplId && i.categoryId !== product.categoryId) {
                        continue
                    }

                    if (!i.productId && i.productId !== companyId) {
                        continue
                    }

                    pricelists.push({
                        pricelistName: p.name,
                        versionName: v.name,
                        productPrice: product.listPrice,
                        productCost: product.standardPrice,
                        ...i
                    })
                }
            }
        }

        return pricelists
    },
    pricelistsForCustomer(state, getters) {
        let customer = getters.selectedCustomer

        if (!customer) {
            return []
        }
        
        let pricelists = []

        for (let p of state.pricelists) {
            if (p.type !== 'sale') {
                continue
            }

            if (p.id !== customer.pricelistId) {
                continue
            }

            pricelists.push(p)
        }

        return pricelists
    },
    showPricelists(state) {
        return state.showPricelists
    }
}

const mutations = {
    setLoadingPricelists(state, loading) {
        state.loadingPricelists = !!loading
    },
    setPricelists(state, pricelists) {
        state.pricelists = pricelists
    },
    setCustomerPricelist(state, pricelistId) {
        let pricelist = null

        for (let p of state.pricelists) {
            if (p.id === pricelistId) {
                pricelist = p
                break
            }
        }

        state.customerPricelist = pricelist
    },
    autoselectPricelist(state) {
        if (state.pricelists.length === 0) {
            return
        }

        state.customerPricelist = state.pricelists[0]
    },
    togglePricelists(state) {
        state.showPricelists = !state.showPricelists
    }
}

const actions = {
    initPricelists({ commit }, pricelists) {
        commit('setPricelists', pricelists)
        commit('autoselectPricelist')
        commit('setLoadingPricelists')
    },
    togglePricelists({ commit }) {
        commit('togglePricelists')
    },
    selectCustomerPricelist({ commit }, pricelistId) {
        console.log(pricelistId)
        commit('setCustomerPricelist', pricelistId)
    },
    resetPricelist({ commit }) {
        commit('setLoadingPricelists', true)
        commit('setPricelists', [])
    }
}

export default {
    state,
    getters,
    actions,
    mutations
}